@@ -22,6 +22,7 @@ import com.google.zxing.DecodeHintType;
 import com.google.zxing.FormatException;
 import com.google.zxing.NotFoundException;
 import com.google.zxing.Result;
+import com.google.zxing.ResultMetadataType;
 import com.google.zxing.ResultPoint;
 import com.google.zxing.common.BitArray;
 
@@ -238,6 +239,8 @@ public final class Code128Reader extends OneDReader {
 
     boolean convertFNC1 = hints != null && hints.containsKey(DecodeHintType.ASSUME_GS1);
 
+    int symbologyModifier = 0;
+
     int[] startPatternInfo = findStartPattern(row);
     int startCode = startPatternInfo[2];
 
@@ -339,6 +342,11 @@ public final class Code128Reader extends OneDReader {
             }
             switch (code) {
               case CODE_FNC_1:
+                if (result.length() == 0) { // FNC1 at first or second character determines the symbology
+                  symbologyModifier = 1;
+                } else if (result.length() == 1) {
+                  symbologyModifier = 2;
+                }
                 if (convertFNC1) {
                   if (result.length() == 0) {
                     // GS1 specification 5.4.3.7. and 5.4.6.4. If the first char after the start code
@@ -351,6 +359,8 @@ public final class Code128Reader extends OneDReader {
                 }
                 break;
               case CODE_FNC_2:
+                symbologyModifier = 4;
+                break;
               case CODE_FNC_3:
                 // do nothing?
                 break;
@@ -395,6 +405,11 @@ public final class Code128Reader extends OneDReader {
             }
             switch (code) {
               case CODE_FNC_1:
+                if (result.length() == 0) { // FNC1 at first or second character determines the symbology
+                  symbologyModifier = 1;
+                } else if (result.length() == 1) {
+                  symbologyModifier = 2;
+                }
                 if (convertFNC1) {
                   if (result.length() == 0) {
                     // GS1 specification 5.4.3.7. and 5.4.6.4. If the first char after the start code
@@ -407,6 +422,8 @@ public final class Code128Reader extends OneDReader {
                 }
                 break;
               case CODE_FNC_2:
+                symbologyModifier = 4;
+                break;
               case CODE_FNC_3:
                 // do nothing?
                 break;
@@ -449,6 +466,11 @@ public final class Code128Reader extends OneDReader {
             }
             switch (code) {
               case CODE_FNC_1:
+                if (result.length() == 0) { // FNC1 at first or second character determines the symbology
+                  symbologyModifier = 1;
+                } else if (result.length() == 1) {
+                  symbologyModifier = 2;
+                }
                 if (convertFNC1) {
                   if (result.length() == 0) {
                     // GS1 specification 5.4.3.7. and 5.4.6.4. If the first char after the start code
@@ -525,14 +547,15 @@ public final class Code128Reader extends OneDReader {
     for (int i = 0; i < rawCodesSize; i++) {
       rawBytes[i] = rawCodes.get(i);
     }
-
-    return new Result(
+    Result resultObject = new Result(
         result.toString(),
         rawBytes,
         new ResultPoint[]{
             new ResultPoint(left, rowNumber),
             new ResultPoint(right, rowNumber)},
         BarcodeFormat.CODE_128);
+    resultObject.putMetadata(ResultMetadataType.SYMBOLOGY_IDENTIFIER, "]C" + symbologyModifier);
+    return resultObject;
 
   }
 
