@@ -26,13 +26,10 @@ final class ExpandedRow {
 
   private final List<ExpandedPair> pairs;
   private final int rowNumber;
-  /** Did this row of the image have to be reversed (mirrored) to recognize the pairs? */
-  private final boolean wasReversed;
 
-  ExpandedRow(List<ExpandedPair> pairs, int rowNumber, boolean wasReversed) {
+  ExpandedRow(List<ExpandedPair> pairs, int rowNumber) {
     this.pairs = new ArrayList<>(pairs);
     this.rowNumber = rowNumber;
-    this.wasReversed = wasReversed;
   }
 
   List<ExpandedPair> getPairs() {
@@ -61,12 +58,12 @@ final class ExpandedRow {
       return false;
     }
     ExpandedRow that = (ExpandedRow) o;
-    return this.pairs.equals(that.pairs) && wasReversed == that.wasReversed;
+    return this.pairs.equals(that.pairs);
   }
 
   @Override
   public int hashCode() {
-    return pairs.hashCode() ^ Boolean.valueOf(wasReversed).hashCode();
+    return pairs.hashCode();
   }
 
 }
