@@ -31,6 +31,7 @@ import com.google.zxing.DecodeHintType;
 import com.google.zxing.FormatException;
 import com.google.zxing.NotFoundException;
 import com.google.zxing.Result;
+import com.google.zxing.ResultMetadataType;
 import com.google.zxing.ResultPoint;
 import com.google.zxing.common.BitArray;
 import com.google.zxing.common.detector.MathUtils;
@@ -58,6 +59,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
   private static final int[] EVEN_TOTAL_SUBSET = {4, 20, 52, 104, 204};
   private static final int[] GSUM = {0, 348, 1388, 2948, 3988};
 
+  /** Finder pattern element widths, from section 7.2.7 of ISO/IEC 24724:2006. */
   private static final int[][] FINDER_PATTERNS = {
     {1,8,4,1}, // A
     {3,6,4,1}, // B
@@ -67,6 +69,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
     {2,2,9,1}  // F
   };
 
+  /** The element weights used in the checksum calculation, from section 7.2.6 of ISO/IEC 24724:2006. */
   private static final int[][] WEIGHTS = {
     {  1,   3,   9,  27,  81,  32,  96,  77},
     { 20,  60, 180, 118, 143,   7,  21,  63},
@@ -100,6 +103,8 @@ public final class RSSExpandedReader extends AbstractRSSReader {
   private static final int FINDER_PAT_E = 4;
   private static final int FINDER_PAT_F = 5;
 
+  /** The possible finder pattern sequences, from section 7.2.7 of ISO/IEC 24724:2006. */
+  @SuppressWarnings("checkstyle:lineLength")
   private static final int[][] FINDER_PATTERN_SEQUENCES = {
     { FINDER_PAT_A, FINDER_PAT_A },
     { FINDER_PAT_A, FINDER_PAT_B, FINDER_PAT_B },
@@ -115,6 +120,10 @@ public final class RSSExpandedReader extends AbstractRSSReader {
 
   private static final int MAX_PAIRS = 11;
 
+  private static final float FINDER_PATTERN_MODULES = 15f;
+  private static final float DATA_CHARACTER_MODULES = 17f;
+  private static final float MAX_FINDER_PATTERN_DISTANCE_VARIANCE = 0.1f;
+
   private final List<ExpandedPair> pairs = new ArrayList<>(MAX_PAIRS);
   private final List<ExpandedRow> rows = new ArrayList<>();
   private final int [] startEnd = new int[2];
@@ -124,9 +133,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
   public Result decodeRow(int rowNumber,
                           BitArray row,
                           Map<DecodeHintType,?> hints) throws NotFoundException, FormatException {
-    // Rows can start with even pattern in case in prev rows there where odd number of patters.
-    // So lets try twice
-    this.pairs.clear();
+    // Rows can start with even pattern if previous rows had an odd number of patterns, so we try twice.
     this.startFromEven = false;
     try {
       return constructResult(decodeRow2pairs(rowNumber, row));
@@ -134,7 +141,6 @@ public final class RSSExpandedReader extends AbstractRSSReader {
       // OK
     }
 
-    this.pairs.clear();
     this.startFromEven = true;
     return constructResult(decodeRow2pairs(rowNumber, row));
   }
@@ -147,6 +153,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
 
   // Not private for testing
   List<ExpandedPair> decodeRow2pairs(int rowNumber, BitArray row) throws NotFoundException {
+    this.pairs.clear();
     boolean done = false;
     while (!done) {
       try {
@@ -160,8 +167,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
       }
     }
 
-    // TODO: verify sequence of finder patterns as in checkPairSequence()
-    if (checkChecksum()) {
+    if (checkChecksum() && isValidSequence(this.pairs, true)) {
       return this.pairs;
     }
 
@@ -222,7 +228,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
       }
       this.pairs.addAll(row.getPairs());
 
-      if (isValidSequence(this.pairs)) {
+      if (isValidSequence(this.pairs, false)) {
         if (checkChecksum()) {
           return this.pairs;
         }
@@ -241,11 +247,12 @@ public final class RSSExpandedReader extends AbstractRSSReader {
     throw NotFoundException.getNotFoundInstance();
   }
 
-  // Whether the pairs form a valid find pattern sequence,
-  // either complete or a prefix
-  private static boolean isValidSequence(List<ExpandedPair> pairs) {
+  // Whether the pairs form a valid finder pattern sequence, either complete or a prefix
+  private static boolean isValidSequence(List<ExpandedPair> pairs, boolean complete) {
+
     for (int[] sequence : FINDER_PATTERN_SEQUENCES) {
-      if (pairs.size() <= sequence.length) {
+      boolean sizeOk = (complete ? pairs.size() == sequence.length : pairs.size() <= sequence.length);
+      if (sizeOk) {
         boolean stop = true;
         for (int j = 0; j < pairs.size(); j++) {
           if (pairs.get(j).getFinderPattern().getValue() != sequence[j]) {
@@ -257,12 +264,47 @@ public final class RSSExpandedReader extends AbstractRSSReader {
           return true;
         }
       }
-
     }
 
     return false;
   }
 
+  // Whether the pairs, plus another pair of the specified type, would together
+  // form a valid finder pattern sequence, either complete or partial
+  private static boolean mayFollow(List<ExpandedPair> pairs, int value) {
+
+    if (pairs.isEmpty()) {
+      return true;
+    }
+
+    for (int[] sequence : FINDER_PATTERN_SEQUENCES) {
+      if (pairs.size() + 1 <= sequence.length) {
+        // the proposed sequence (i.e. pairs + value) would fit in this allowed sequence
+        for (int i = pairs.size(); i < sequence.length; i++) {
+          if (sequence[i] == value) {
+            // we found our value in this allowed sequence, check to see if the elements preceding it match our existing
+            // pairs; note our existing pairs may not be a full sequence (e.g. if processing a row in a stacked symbol)
+            boolean matched = true;
+            for (int j = 0; j < pairs.size(); j++) {
+              int allowed = sequence[i - j - 1];
+              int actual = pairs.get(pairs.size() - j - 1).getFinderPattern().getValue();
+              if (allowed != actual) {
+                matched = false;
+                break;
+              }
+            }
+            if (matched) {
+              return true;
+            }
+          }
+        }
+      }
+    }
+
+    // the proposed finder pattern sequence is illegal
+    return false;
+  }
+
   private void storeRow(int rowNumber) {
     // Discard if duplicate above or below; otherwise insert in order by row number.
     int insertPos = 0;
@@ -290,7 +332,7 @@ public final class RSSExpandedReader extends AbstractRSSReader {
       return;
     }
 
-    this.rows.add(insertPos, new ExpandedRow(this.pairs, rowNumber, false));
+    this.rows.add(insertPos, new ExpandedRow(this.pairs, rowNumber));
 
     removePartialRows(this.pairs, this.rows);
   }
@@ -355,12 +397,14 @@ public final class RSSExpandedReader extends AbstractRSSReader {
     ResultPoint[] firstPoints = pairs.get(0).getFinderPattern().getResultPoints();
     ResultPoint[] lastPoints  = pairs.get(pairs.size() - 1).getFinderPattern().getResultPoints();
 
-    return new Result(
+    Result result = new Result(
           resultingString,
           null,
           new ResultPoint[]{firstPoints[0], firstPoints[1], lastPoints[0], lastPoints[1]},
           BarcodeFormat.RSS_EXPANDED
       );
+    result.putMetadata(ResultMetadataType.SYMBOLOGY_IDENTIFIER, "]e0");
+    return result;
   }
 
   private boolean checkChecksum() {
@@ -414,24 +458,28 @@ public final class RSSExpandedReader extends AbstractRSSReader {
     }
 
     FinderPattern pattern;
+    DataCharacter leftChar = null;
 
     boolean keepFinding = true;
     int forcedOffset = -1;
     do {
       this.findNextPair(row, previousPairs, forcedOffset);
-      pattern = parseFoundFinderPattern(row, rowNumber, isOddPattern);
+      pattern = parseFoundFinderPattern(row, rowNumber, isOddPattern, previousPairs);
       if (pattern == null) {
-        forcedOffset = getNextSecondBar(row, this.startEnd[0]);
+        forcedOffset = getNextSecondBar(row, this.startEnd[0]); // probable false positive, keep looking
       } else {
-        keepFinding = false;
+        try {
+          leftChar  = this.decodeDataCharacter(row, pattern, isOddPattern, true);
+          keepFinding = false;
+        } catch (NotFoundException ignored) {
+          forcedOffset = getNextSecondBar(row, this.startEnd[0]); // probable false positive, keep looking
+        }
       }
     } while (keepFinding);
 
     // When stacked symbol is split over multiple rows, there's no way to guess if this pair can be last or not.
     // boolean mayBeLast = checkPairSequence(previousPairs, pattern);
 
-    DataCharacter leftChar  = this.decodeDataCharacter(row, pattern, isOddPattern, true);
-
     if (!previousPairs.isEmpty() && previousPairs.get(previousPairs.size() - 1).mustBeLast()) {
       throw NotFoundException.getNotFoundInstance();
     }
@@ -524,7 +572,10 @@ public final class RSSExpandedReader extends AbstractRSSReader {
     }
   }
 
-  private FinderPattern parseFoundFinderPattern(BitArray row, int rowNumber, boolean oddPattern) {
+  private FinderPattern parseFoundFinderPattern(BitArray row,
+                                                int rowNumber,
+                                                boolean oddPattern,
+                                                List<ExpandedPair> previousPairs) {
     // Actually we found elements 2-5.
     int firstCounter;
     int start;
@@ -564,6 +615,27 @@ public final class RSSExpandedReader extends AbstractRSSReader {
     } catch (NotFoundException ignored) {
       return null;
     }
+
+    // Check that the pattern type that we *think* we found can exist as part of a valid sequence of finder patterns.
+    if (!mayFollow(previousPairs, value)) {
+      return null;
+    }
+
+    // Check that the finder pattern that we *think* we found is not too far from where we would expect to find it,
+    // given that finder patterns are 15 modules wide and the data characters between them are 17 modules wide.
+    if (!previousPairs.isEmpty()) {
+      ExpandedPair prev = previousPairs.get(previousPairs.size() - 1);
+      int prevStart = prev.getFinderPattern().getStartEnd()[0];
+      int prevEnd = prev.getFinderPattern().getStartEnd()[1];
+      int prevWidth = prevEnd - prevStart;
+      float charWidth = (prevWidth / FINDER_PATTERN_MODULES) * DATA_CHARACTER_MODULES;
+      float minX = prevEnd + (2 * charWidth * (1 - MAX_FINDER_PATTERN_DISTANCE_VARIANCE));
+      float maxX = prevEnd + (2 * charWidth * (1 + MAX_FINDER_PATTERN_DISTANCE_VARIANCE));
+      if (start < minX || start > maxX) {
+        return null;
+      }
+    }
+
     return new FinderPattern(value, new int[] {start, end}, start, end, rowNumber);
   }
 
