@@ -28,10 +28,16 @@ public final class PDF417DetectorResult {
 
   private final BitMatrix bits;
   private final List<ResultPoint[]> points;
+  private final int rotation;
 
-  public PDF417DetectorResult(BitMatrix bits, List<ResultPoint[]> points) {
+  public PDF417DetectorResult(BitMatrix bits, List<ResultPoint[]> points, int rotation) {
     this.bits = bits;
     this.points = points;
+    this.rotation = rotation;
+  }
+
+  public PDF417DetectorResult(BitMatrix bits, List<ResultPoint[]> points) {
+    this(bits, points, 0);
   }
 
   public BitMatrix getBits() {
@@ -42,4 +48,8 @@ public final class PDF417DetectorResult {
     return points;
   }
 
+  public int getRotation() {
+    return rotation;
+  }
+
 }
