@@ -643,10 +643,20 @@ public final class PDF417 {
    * @throws WriterException if the contents cannot be encoded in this format
    */
   public void generateBarcodeLogic(String msg, int errorCorrectionLevel) throws WriterException {
+    generateBarcodeLogic(msg, errorCorrectionLevel, false);
+  }
+
+  /**
+   * @param msg message to encode
+   * @param errorCorrectionLevel PDF417 error correction level to use
+   * @param autoECI automatically insert ECIs if needed
+   * @throws WriterException if the contents cannot be encoded in this format
+   */
+  public void generateBarcodeLogic(String msg, int errorCorrectionLevel, boolean autoECI) throws WriterException {
 
     //1. step: High-level encoding
     int errorCorrectionCodeWords = PDF417ErrorCorrection.getErrorCorrectionCodewordCount(errorCorrectionLevel);
-    String highLevel = PDF417HighLevelEncoder.encodeHighLevel(msg, compaction, encoding);
+    String highLevel = PDF417HighLevelEncoder.encodeHighLevel(msg, compaction, encoding, autoECI);
     int sourceCodeWords = highLevel.length();
 
     int[] dimension = determineDimensions(sourceCodeWords, errorCorrectionCodeWords);
@@ -713,13 +723,13 @@ public final class PDF417 {
       dimension = new int[] {cols, rows};
     }
 
-     // Handle case when min values were larger than necessary
-     if (dimension == null) {
-       int rows = calculateNumberOfRows(sourceCodeWords, errorCorrectionCodeWords, minCols);
-       if (rows < minRows) {
-         dimension = new int[]{minCols, minRows};
-       }
-     }
+    // Handle case when min values were larger than necessary
+    if (dimension == null) {
+      int rows = calculateNumberOfRows(sourceCodeWords, errorCorrectionCodeWords, minCols);
+      if (rows < minRows) {
+        dimension = new int[]{minCols, minRows};
+      }
+    }
 
     if (dimension == null) {
       throw new WriterException("Unable to fit message in columns");
