@@ -62,6 +62,7 @@ public abstract class AbstractBlackBoxTestCase extends Assert {
   private final Reader barcodeReader;
   private final BarcodeFormat expectedFormat;
   private final List<TestResult> testResults;
+  private final EnumMap<DecodeHintType,Object> hints = new EnumMap<>(DecodeHintType.class);
 
   public static Path buildTestBase(String testBasePathSuffix) {
     // A little workaround to prevent aggravation in my IDE
@@ -92,6 +93,10 @@ public abstract class AbstractBlackBoxTestCase extends Assert {
     addTest(mustPassCount, tryHarderCount, 0, 0, rotation);
   }
 
+  protected void addHint(DecodeHintType hint) {
+    hints.put(hint, Boolean.TRUE);
+  }
+
   /**
    * Adds a new test for the current directory of images.
    *
@@ -160,6 +165,8 @@ public abstract class AbstractBlackBoxTestCase extends Assert {
         try (BufferedReader reader = Files.newBufferedReader(expectedMetadataFile, StandardCharsets.UTF_8)) {
           expectedMetadata.load(reader);
         }
+        correctInteger(expectedMetadata, ResultMetadataType.ERRORS_CORRECTED);
+        correctInteger(expectedMetadata, ResultMetadataType.ERASURES_CORRECTED);
       }
 
       for (int x = 0; x < testCount; x++) {
@@ -244,6 +251,15 @@ public abstract class AbstractBlackBoxTestCase extends Assert {
     }
   }
 
+  private static void correctInteger(Properties metadata, ResultMetadataType key) {
+    String skey = key.toString();
+    if (metadata.containsKey(skey)) {
+      String sval = metadata.getProperty(skey);
+      Integer ival = Integer.parseInt(sval);
+      metadata.put(skey, ival);
+    }
+  }
+
   private boolean decode(BinaryBitmap source,
                          float rotation,
                          String expectedText,
@@ -252,7 +268,7 @@ public abstract class AbstractBlackBoxTestCase extends Assert {
 
     String suffix = String.format(" (%srotation: %d)", tryHarder ? "try harder, " : "", (int) rotation);
 
-    Map<DecodeHintType,Object> hints = new EnumMap<>(DecodeHintType.class);
+    Map<DecodeHintType,Object> hints = this.hints.clone();
     if (tryHarder) {
       hints.put(DecodeHintType.TRY_HARDER, Boolean.TRUE);
     }
