@@ -16,8 +16,8 @@
 
 package com.google.zxing.web;
 
-import javax.servlet.annotation.WebFilter;
-import javax.servlet.annotation.WebInitParam;
+import jakarta.servlet.annotation.WebFilter;
+import jakarta.servlet.annotation.WebInitParam;
 
 /**
  * Protect the /chart endpoint from too many requests.
@@ -25,7 +25,7 @@ import javax.servlet.annotation.WebInitParam;
 @WebFilter(urlPatterns = {"/w/chart"}, initParams = {
   @WebInitParam(name = "maxAccessPerTime", value = "120"),
   @WebInitParam(name = "accessTimeSec", value = "60"),
-  @WebInitParam(name = "maxEntries", value = "10000"),
+  @WebInitParam(name = "maxEntries", value = "100000"),
   @WebInitParam(name = "maxLoad", value = "0.9")
 })
 public final class ChartDoSFilter extends DoSFilter {
