@@ -47,8 +47,8 @@ final class DoSTracker {
    *                exceeds this value, and upwards when below this value
    */
   DoSTracker(Timer timer,
-             final String name,
-             final int maxAccessesPerTime,
+             String name,
+             int maxAccessesPerTime,
              long accessTimeMS,
              int maxEntries,
              Double maxLoad) {
@@ -89,7 +89,10 @@ final class DoSTracker {
       // smallest count > maxAccessesPerTime
       int minDisallowedCount = Integer.MAX_VALUE;
       int localMAPT = maxAccessesPerTime;
+      int totalEntries;
+      int clearedEntries = 0;
       synchronized (numRecentAccesses) {
+        totalEntries = numRecentAccesses.size();
         Iterator<Map.Entry<String,AtomicInteger>> accessIt = numRecentAccesses.entrySet().iterator();
         while (accessIt.hasNext()) {
           Map.Entry<String,AtomicInteger> entry = accessIt.next();
@@ -99,15 +102,16 @@ final class DoSTracker {
           if (count <= localMAPT) {
             accessIt.remove();
             maxAllowedCount = Math.max(maxAllowedCount, count);
+            clearedEntries++;
           } else {
-            // Else it exceeded the max, so log it (again)
-            log.warning(name + ": Blocking " + entry.getKey() + " (" + count + " outstanding)");
             // Reduce count of accesses held against the host
             atomicCount.getAndAdd(-localMAPT);
             minDisallowedCount = Math.min(minDisallowedCount, count);
           }
         }
       }
+      log.info(name + ": " + clearedEntries + " of " + totalEntries + " cleared");
+
       if (maxLoad != null) {
         OperatingSystemMXBean mxBean = ManagementFactory.getOperatingSystemMXBean();
         if (mxBean == null) {
@@ -117,11 +121,13 @@ final class DoSTracker {
           if (loadAvg >= 0.0) {
             int cores = mxBean.getAvailableProcessors();
             double loadRatio = loadAvg / cores;
-            log.info(name + ": Load ratio: " + loadRatio + " (" + loadAvg + '/' + cores + ") vs " + maxLoad);
-            maxAccessesPerTime = loadRatio > maxLoad ?
-              Math.min(maxAllowedCount, maxAccessesPerTime) :
-              Math.max(minDisallowedCount, maxAccessesPerTime);
-            log.info(name + ": New maxAccessesPerTime: " + maxAccessesPerTime);
+            int newMaxAccessesPerTime = loadRatio > maxLoad ?
+                Math.min(maxAllowedCount, Math.max(1, maxAccessesPerTime - 1)) :
+                Math.max(minDisallowedCount, maxAccessesPerTime);
+            log.info(name + ": Load ratio: " + loadRatio +
+                " (" + loadAvg + '/' + cores + ") vs " + maxLoad +
+                " ; new maxAccessesPerTime: " + newMaxAccessesPerTime);
+            maxAccessesPerTime = newMaxAccessesPerTime;
           }
         }
       }
